# Generated by Django 5.0.12 on 2025-02-20 14:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbfdm', '0029_alter_instrumentprice_volume'),
    ]

    operations = [
        migrations.AlterField(
            model_name='relatedinstrumentthroughmodel',
            name='related_type',
            field=models.CharField(choices=[('BENCHMARK', 'Benchmark'), ('PEER', 'Peer'), ('RISK_INSTRUMENT', 'Risk Instrument'), ('TRACKER', 'Tracker')], default='BENCHMARK', max_length=32),
        ),
    ]
