# Generated by Django 4.2.11 on 2024-04-03 08:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0015_instrument_instrument_investible_index"),
    ]

    operations = [
        migrations.AddField(
            model_name="instrumenttype",
            name="name_repr",
            field=models.CharField(default="name", max_length=128, verbose_name="Name (Representation)"),
            preserve_default=False,
        ),
    ]
