# Generated by Django 4.2.11 on 2024-04-05 07:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0016_instrumenttype_name_repr"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="instrument",
            index=models.Index(fields=["is_security"], name="instrument_security_index"),
        ),
    ]
